@@ -22,7 +22,7 @@ Puppet::Type.type(:service).provide :base, :parent => :service do
       'ps auxwww'
     else
       'ps -ef'
-     end
+    end
   end
   private :getps
 
@@ -32,15 +32,28 @@ Puppet::Type.type(:service).provide :base, :parent => :service do
     @resource.fail "Either stop/status commands or a pattern must be specified" unless @resource[:pattern]
     regex = Regexp.new(@resource[:pattern])
     ps = getps
+
     self.debug "Executing '#{ps}'"
-    IO.popen(ps) { |table|
-      table.each_line { |line|
-        if regex.match(line)
-          self.debug "Process matched: #{line}"
-          ary = line.sub(/^\s+/, '').split(/\s+/)
-          return ary[1]
-        end
-      }
+    table = Puppet::Util::Execution.execute(ps)
+
+    # The output of the PS command can be a mashup of several different
+    # encodings depending on which processes are running and what
+    # arbitrary data has been used to set their name in the process table.
+    #
+    # First, try a polite conversion to in order to match the UTF-8 encoding
+    # of our regular expression.
+    table = Puppet::Util::CharacterEncoding.convert_to_utf_8(table)
+    # If that fails, force to UTF-8 and then scrub as most uses are scanning
+    # for ACII-compatible program names.
+    table.force_encoding(Encoding::UTF_8) unless table.encoding == Encoding::UTF_8
+    table = table.scrub unless table.valid_encoding?
+
+    table.each_line { |line|
+      if regex.match(line)
+        self.debug "Process matched: #{line}"
+        ary = line.sub(/^[[:space:]]+/u, '').split(/[[:space:]]+/u)
+        return ary[1]
+      end
     }
 
     nil
@@ -63,11 +76,14 @@ Puppet::Type.type(:service).provide :base, :parent => :service do
       else
         return :stopped
       end
-    elsif pid = getpid
-      self.debug "PID is #{pid}"
-      return :running
     else
-      return :stopped
+      pid = getpid
+      if pid
+        self.debug "PID is #{pid}"
+        return :running
+      else
+        return :stopped
+      end
     end
   end
 
@@ -102,7 +118,7 @@ Puppet::Type.type(:service).provide :base, :parent => :service do
     else
       pid = getpid
       unless pid
-        self.info "#{self.name} is not running"
+        self.info _("%{name} is not running") % { name: self.name }
         return false
       end
       begin
