@@ -9,7 +9,7 @@ module Puppet
       On Windows, a group (such as "Administrators") can be set as a file's owner
       and a user (such as "Administrator") can be set as a file's group; however,
       a file's owner and group shouldn't be the same. (If the owner is also
-      the group, files with modes like `0640` will cause log churn, as they
+      the group, files with modes like `"0640"` will cause log churn, as they
       will always appear out of sync.)
     EOT
 
@@ -33,11 +33,11 @@ module Puppet
 
     # We want to print names, not numbers
     def is_to_s(currentvalue)
-      provider.uid2name(currentvalue) || currentvalue
+      super(provider.uid2name(currentvalue) || currentvalue)
     end
 
     def should_to_s(newvalue)
-      provider.uid2name(newvalue) || newvalue
+      super(provider.uid2name(newvalue) || newvalue)
     end
   end
 end
